@@ -35,22 +35,22 @@ impl JournalWriter {
 
     pub fn start(&self) -> () {
         for entry in &self.rx {
-            self.write(entry);
+            let result = self.write(entry);
+
+            if let Err(err) = result {
+                panic!("Error while writing to journal: {}", err)
+            }
         }
     }
 
-    fn write(&self, entry: JournalEntry) {
+    fn write(&self, entry: JournalEntry) -> io::Result<()> {
         let log_file = journal_file(&entry, &self.journal_dir);
 
-        let mut f = File::options()
-            .create(true)
-            .append(true)
-            .open(log_file)
-            .unwrap();
+        let mut f = File::options().create(true).append(true).open(log_file)?;
 
         let time = humantime::format_rfc3339_seconds(entry.time);
 
-        writeln!(f, "{} {:+.2}", time, entry.remaining_watt_hours);
+        writeln!(f, "{} {:+.2}", time, entry.remaining_watt_hours)
     }
 }
 
@@ -63,11 +63,19 @@ impl JournalReader {
         info!("Scanning journal for unfinished deliveries");
         for entry in fs::read_dir(&self.journal_dir)? {
             let address = entry?;
-            let path = address.path();
-            debug!("Loading journal for address {:?}", address);
-            for entry2 in fs::read_dir(path)? {
+            let address2 = address.file_name().into_string().unwrap();
+
+            debug!("Loading journal for address {}", address2);
+            for entry2 in fs::read_dir(address.path())? {
                 let txid = entry2?;
-                debug!("Loading txid {:?}", txid);
+                let txid2 = txid
+                    .file_name()
+                    .into_string()
+                    .unwrap()
+                    .trim_end_matches(".log")
+                    .to_string();
+
+                debug!("Loading txid {}", txid2);
 
                 let line = last_line(&txid.path());
                 let remaining_watt_hours: f64 = match line?.split_once(' ') {
@@ -76,13 +84,8 @@ impl JournalReader {
                 };
 
                 let credit = Payment {
-                    address: address.file_name().into_string().unwrap(),
-                    txid: txid
-                        .file_name()
-                        .into_string()
-                        .unwrap()
-                        .trim_end_matches(".log")
-                        .to_string(),
+                    address: address2.clone(),
+                    txid: txid2,
                     watt_hours: remaining_watt_hours,
                 };
 
