@@ -1,8 +1,6 @@
 #[macro_use]
 extern crate log;
 
-use serde::Deserialize;
-use serde_json::json;
 use std::collections::HashMap;
 use std::collections::HashSet;
 
@@ -16,32 +14,15 @@ mod args;
 mod common;
 mod config;
 mod journal;
+mod monero;
 mod shelly;
 
 use crate::args::Args;
 use crate::common::Payment;
-use crate::config::{Config, Device, HostPort, Price};
+use crate::config::{Config, Device, Price};
 use crate::journal::{JournalEntry, JournalReader, JournalWriter};
-
-#[derive(Deserialize, Debug, Clone)]
-struct MoneroResponse {
-    result: MoneroResult,
-}
-
-#[derive(Deserialize, Debug, Clone)]
-struct MoneroResult {
-    #[serde(rename = "in")]
-    transfers: Option<Vec<MoneroTransfer>>,
-
-    pool: Option<Vec<MoneroTransfer>>,
-}
-
-#[derive(Deserialize, Debug, Clone)]
-struct MoneroTransfer {
-    address: String,
-    amount: u64,
-    txid: String,
-}
+use crate::monero::{Monero, MoneroTransfer};
+use crate::shelly::Shelly;
 
 fn main() -> () {
     env_logger::init();
@@ -58,7 +39,16 @@ fn main() -> () {
         String::from(journal_dir.to_string_lossy())
     );
 
-    let config: Config = config::load_from_file(&config_file.to_string());
+    let config = match Config::from_file(&config_file) {
+        Ok(config) => {
+            debug!("Loaded config with {} devices", config.device.len());
+            config
+        }
+        Err(err) => {
+            error!("Could not load config file: {}", err);
+            std::process::exit(1);
+        }
+    };
 
     let (journal_tx, journal_rx): (Sender<JournalEntry>, Receiver<JournalEntry>) = mpsc::channel();
 
@@ -79,9 +69,13 @@ fn main() -> () {
         journal.start();
     });
 
+    let monero = Monero {
+        sender: monero_tx,
+        config: config.monero_rpc,
+    };
     thread::spawn(move || loop {
-        match listen_for_monero_payments(monero_tx.clone(), config.monero_rpc.clone()) {
-            Ok(_) => error!("Premature return from Monero query"),
+        match monero.listen_for_payments() {
+            Ok(_) => error!("Pre-mature exit from Monero query"),
             Err(err) => error!("Error while query Monero wallet: {}", err),
         }
 
@@ -126,7 +120,7 @@ fn route_payments(
             Some(channel) => {
                 if credit.watt_hours > 0.0 {
                     info!("Got credit of {} for {} Wh", credit.txid, credit.watt_hours);
-                    channel.send(credit.clone());
+                    channel.send(credit.clone()).unwrap();
                 }
                 processed_transactions.insert(credit.txid.clone());
             }
@@ -147,7 +141,7 @@ fn route_payments(
                     txid: transfer.txid.clone(),
                 };
 
-                channel.send(payment);
+                channel.send(payment).unwrap();
                 processed_transactions.insert(transfer.txid.clone());
             }
             None => error!("missing device for address {}", &transfer.address),
@@ -190,82 +184,17 @@ mod tests {
     }
 }
 
-fn listen_for_monero_payments(
-    sender: Sender<MoneroTransfer>,
-    config: HostPort,
-) -> Result<(), attohttpc::Error> {
-    let one_second = Duration::from_secs(1);
-
-    let mut old_transactions: HashSet<String> = HashSet::new();
-
-    let url = format!("http://{}:{}/json_rpc", config.host, config.port);
-    let refresh = json!({
-        "jsonrpc": "2.0",
-        "id": "0",
-        "method": "refresh",
-        "params": {"start_height": 2598796}
-    });
-
-    let get_transfers = json!({
-        "jsonrpc": "2.0",
-        "id": "1",
-        "method": "get_transfers",
-        "params": {"in":true,"pending":true,"pool":true}
-    });
-
-    info!("Waiting for payments from Monero");
-    loop {
-        attohttpc::post(&url).json(&refresh)?.send()?;
-
-        let res = attohttpc::post(&url).json(&get_transfers)?.send()?;
-
-        let response: MoneroResponse = res.json()?;
-
-        match response.result.transfers {
-            Some(t) => iterate_monero_transactions(&t, &mut old_transactions, &sender),
-            None => (),
-        }
-
-        match response.result.pool {
-            Some(t) => iterate_monero_transactions(&t, &mut old_transactions, &sender),
-            None => (),
-        }
-
-        thread::sleep(one_second);
-    }
-}
-
-fn iterate_monero_transactions(
-    transactions: &Vec<MoneroTransfer>,
-    old_transactions: &mut HashSet<String>,
-    sender: &Sender<MoneroTransfer>,
-) {
-    for t in transactions {
-        if old_transactions.contains(&t.txid) {
-            continue;
-        }
-
-        let xmr = t.amount as f64 / 1_000_000_000_000.0;
-
-        info!("Received {:0.12} XMR to {}", xmr, t.address);
-        let hash = t.txid.clone();
-
-        sender.send(t.clone());
-        old_transactions.insert(hash);
-    }
-}
-
 fn waiting_for_payment_per_device(
     payment_rx: Receiver<Payment>,
     journal: Sender<JournalEntry>,
     device: &Device,
 ) {
     for payment in payment_rx {
-        deliver_electricity(journal.clone(), device, payment);
+        handle_payment(journal.clone(), device, payment);
     }
 }
 
-fn deliver_electricity(journal_tx: Sender<JournalEntry>, device: &Device, payment: Payment) -> () {
+fn handle_payment(journal_tx: Sender<JournalEntry>, device: &Device, payment: Payment) -> () {
     journal_tx
         .send(JournalEntry {
             time: SystemTime::now(),
@@ -277,9 +206,10 @@ fn deliver_electricity(journal_tx: Sender<JournalEntry>, device: &Device, paymen
 
     let ten_seconds = Duration::from_secs(10);
     let mut start: Option<f64> = None;
+    let shelly = Shelly::new(device.clone());
 
     loop {
-        match shelly::status(device) {
+        match shelly.status() {
             Ok(s) => {
                 let total = s.aenergy.total;
                 match start {
@@ -287,7 +217,7 @@ fn deliver_electricity(journal_tx: Sender<JournalEntry>, device: &Device, paymen
                         start = Some(total);
                         info!("{}: Turing on, meter at {:.2} Wh", device.location, total);
                         debug!("{}: Current txid: {}", device.location, payment.txid);
-                        let result = shelly::on(device);
+                        let result = shelly.on();
                         match result {
                             Ok(_) => debug!("{}: Device turned on", device.location),
                             Err(err) => error!(
@@ -314,7 +244,7 @@ fn deliver_electricity(journal_tx: Sender<JournalEntry>, device: &Device, paymen
                         );
 
                         if total < end {
-                            let result = shelly::on(device);
+                            let result = shelly.on();
                             match result {
                                 Ok(_) => {
                                     debug!("{}: Make sure device is turned on", device.location)
@@ -327,7 +257,7 @@ fn deliver_electricity(journal_tx: Sender<JournalEntry>, device: &Device, paymen
                         } else {
                             info!("{}: Turing off, meter at {:.2} Wh", device.location, total);
                             debug!("{}: Current txid: {}", device.location, payment.txid);
-                            let result = shelly::off(device);
+                            let result = shelly.off();
 
                             match result {
                                 Err(err) => error!(
